﻿/**
 *	MiniBatteryStatus Widget
 *
 *	© Claudio Procida 2005-2008
 *
 *	Disclaimer
 *
 *	The MiniBatteryStatus Widget software (from now, the "Software") and the accompanying materials
 *	are provided “AS IS” without warranty of any kind. IN NO EVENT SHALL THE AUTHOR(S) BE
 *	LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 *	INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 *	IF THE AUTHOR(S) HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. The entire risk as to
 *	the results and performance of this software is assumed by you. If the software is
 *	defective, you, and not its author(s), assume the entire cost of all necessary servicing,
 *	repairs and corrections. If you do not agree to these terms and conditions, you may not
 *	install or use this software.
 */

/**
 *
 */
Date.prototype.toSQLString = function()
{
	var comps = [];
	comps.push(this.getFullYear());
	var month = this.getMonth() + 1;
	comps.push(month < 10 ? ("0" + month) : month);
	var day = this.getDate();
	comps.push(day < 10 ? ("0" + day) : day);
	return comps.join("-");
};

/**
 *	Returns true if the array contains the given object.
 */
Array.prototype.contains = function(element)
{
	return this.indexOf(element) != -1;
};

/**
 *	Returns the longest element (in the sense of strings) of the receiver.
 */
Array.prototype.longest = function()
{
	var maxlen = 0;
	var longest = null;
	for (var i = 0; i < this.length; i++)
	{
		if (this[i].length > maxlen)
		{
			maxlen = this[i].length;
			longest = this[i];
		}
	}
	return longest;
};

/**
 *	Returns the shortest element (in the sense of strings) of the receiver.
 */
Array.prototype.shortest = function()
{
	var minlen = Infinity;
	var shortest = null;
	for (var i = 0; i < this.length; i++)
	{
		if (this[i].length < minlen)
		{
			minlen = this[i].length;
			shortest = this[i];
		}
	}
	return shortest;
};

/**
 *	Returns the position of a given element in the receiver.
 */
Array.prototype.indexOf = function(el)
{
	for (var i = 0; i < this.length; i++)
	{
		if (this[i] == el)
		{
			return i;
		}
	}
	return -1;
};

/**
 *	Replaces newlines with line breaks
 */
String.prototype.nl2br = function()
{
	return this.replace(/(\r\n|\r|\n)/g, "<br />");
};

/**
 *	Splits a query string into a dictionary of keys and values
 */
String.prototype.queryStringDictionary = function()
{
	var pairs = this.split("&");
	var dict = {};
	for (var i = 0; i < pairs.length; i++)
	{
		var keyval = pairs[i].split("=");
		dict[keyval[0]] = decodeURIComponent(keyval[1]);
	}
	return dict;
};

/**
 *	@method sprintf
 *	@abstract Returns a formatted string by replacing the placeholders in the receivers
 *	with the arguments.
 *	@discussion This method will replace every occurrence of <tt>%s</tt> with its arguments.
 *	@param ... Variable length list of replacements.
 *	@result The formatted string.
 */
String.prototype.sprintf = function()
{
	var str = this;
	for (var i = 0; i < arguments.length; i++)
	{
		str = str.replace("%s", arguments[i]);
	}
	return str;
};

/**
 *	Transforms a dictionary into a query string
 */
Object.prototype.toQueryString = function()
{
	var qs = "";
	for (var i in this)
	{
		if (["string","number","boolean"].contains(typeof this[i]))
		{
			qs += (qs ? "&" : "") + i + "=" + encodeURIComponent(this[i]);
		}
		else if (this[i] instanceof Date)
		{
			qs += (qs ? "&" : "") + i + "=" + encodeURIComponent(this[i].toSQLString());
		}
	}
	return qs;
};

/**
 *	Returns all the values of a dictionary into an array.
 *	By default, only primitive values are returned.
 */
Object.prototype.allValues = function()
{
	var values = [];
	for (var i in this)
	{
		if (["string","number","boolean"].contains(typeof this[i]))
		{
			values.push(this[i]);
		}
	}
	return values;
};

/**
 *	Returns all the keys of a dictionary into an array.
 *	By default, only primitive values are returned.
 */
Object.prototype.allKeys = function()
{
	var keys = [];
	for (var i in this)
	{
		if (["string","number","boolean"].contains(typeof this[i]))
		{
			keys.push(i);
		}
	}
	return keys;
};